package argonavis.pdf;

import java.io.*;

import org.apache.tools.ant.*;

import org.apache.fop.messaging.MessageHandler;
import org.apache.fop.apps.*;
import org.apache.log.*;
import org.xml.sax.InputSource;

/**
 * This Ant task uses Apache FOP 0.20 (xml.apache.org) to
 * transform XSL-FO documents into PDF documents
 */
public class FopTask extends Task {
    
    private File sourceDir, destDir, in, out;
    private String includes = ".fo";
    
    /**
     * For each FO file in sourceDir, generate a
     * PDF file and store it in destDir.
     * If PDF generation fails, skip to next file.
     * If all transformations fail, the task should fail.
     */
    public void execute() throws BuildException {
        if (in != null) {
            executeInOut();
        } else if (sourceDir != null) {  
            executeSrcDest();
        }
    }
    
    public void executeSrcDest() throws BuildException {
        if (destDir == null) {
            throw new BuildException("The 'baseDir' attribute requires a matching 'destDir' attribute.");
        }
        String[] fileNames = sourceDir.list(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                if(name.indexOf(includes) != -1) {
                    return true;
                }
                return false;
            }
        });
        if (fileNames == null || fileNames.length == 0) {
            throw new BuildException("No files to transform!");
        }

        boolean oneWorked = false;
        for (int i = 0; i < fileNames.length; i++) {
            System.out.println("Processing " + fileNames[i]);
            int extDot = fileNames[i].lastIndexOf(".");
            String rootName = fileNames[i].substring(0, extDot);
            File destFile = new File(destDir, rootName + ".pdf");
            File srcFile = new File(sourceDir, fileNames[i]);
            try {
                generatePDF(srcFile, destFile);
                oneWorked = true;
            } catch (IOException e) {
                System.out.println("Generation of " + rootName + ".pdf from "+fileNames[i]+" failed!");
                continue;
            }
        }
        if (!oneWorked) { 
            throw new BuildException("No PDF files were generated!");
        }    
    }
    
    public void executeInOut() throws BuildException {
        if (out == null) {
            throw new BuildException("The 'in' attribute requires a matching 'out' attribute.");
        }
        try {
            generatePDF(in, out);
        } catch (IOException e) {
            throw new BuildException ("Generation of " + out + " from "+in+" failed!");
        }
    }
    
    /**
     * Defines the basedir attribute
     */
    public void setBasedir(File sourceDir) throws BuildException {
        if (!sourceDir.isDirectory() || !sourceDir.exists()) {
            throw new BuildException("Source directory " + sourceDir + " does not exist!");
        }
        this.sourceDir = sourceDir;
    }
    
    /**
     * Defines the destdir attribute
     */
    public void setDestdir(File destDir) throws BuildException {
        if (!destDir.isDirectory() || !destDir.exists()) {
            throw new BuildException("Destination directory " + destDir + " does not exist!");
        }
        this.destDir = destDir;
    }
    
    /**
     * Defines the in attribute
     */
    public void setIn(File inFile) throws BuildException {
        if (!inFile.isFile() || !inFile.exists()) {
            throw new BuildException("Source file " + inFile + " does not exist!");
        }
        this.in = inFile;
    }
    
    /**
     * Defines the out attribute
     */
    public void setOut(File outFile) throws BuildException {
        this.out = outFile;
    }
    
    /**
     * This method generates a PDF file from a XSL-FO styled file
     * using Apache FOP libraries.
     */
    public static void generatePDF(File foFile, File pdfFile) throws IOException {
        FileOutputStream out = new FileOutputStream(pdfFile);
        InputSource in = new InputSource(new FileInputStream(foFile));
        
        Hierarchy hierarchy = Hierarchy.getDefaultHierarchy();
        Logger log = hierarchy.getLoggerFor("fop");
        log.setPriority(Priority.WARN);
	 	Driver driver = new Driver();
        
		driver.setRenderer(Driver.RENDER_PDF);
		driver.setInputSource(in);
		driver.setOutputStream(out);
        try {
		    driver.run();
        } catch (FOPException e) {
            throw new IOException("Unable to run FOP driver!");
        }
	}
    
    /**
     * Only for testing. Package-level access.
     */
    File getBasedir() { return sourceDir; }
    /**
     * Only for testing. Package-level access.
     */
    File getDestdir() { return destDir; }
    /**
     * Only for testing. Package-level access.
     */
    File getIn() { return in; }
    /**
     * Only for testing. Package-level access.
     */
    File getOut() { return out; }
    
}
